﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Devices.Gpio;

namespace GoBusExperimental
{
    public class GoHub : GoHubBase, IDisposable
    {
        private bool _disposed;

        public void AddGoPort(IGoPort port, int ledPinNumber)
        {
            _goPorts.Add(port);
            port.Hub = this;

            var gpioController = GpioController.GetDefault();

            if (ledPinNumber >= 0)
            {
                var led = gpioController.OpenPin(ledPinNumber);
                led.SetDriveMode(GpioPinDriveMode.Output);
                _portLEDs.Add(led);
            }
            else
            {
                _portLEDs.Add(null);
            }

            port.Initialize();
        }

        #region Port LEDs

        private readonly List<GpioPin> _portLEDs = new List<GpioPin>();

        protected virtual void SetPortLEDState(IGoPort port, bool state)
        {
            var index = _goPorts.IndexOf(port);
            var led = _portLEDs[index];
            if (led == null)
                return;

            led.Write((state) ? GpioPinValue.Low : GpioPinValue.High);
        }

        #endregion

        public override void OnGoPortBound(IGoPort goPort)
        {
            SetPortLEDState(goPort, true);
        }

        public override void OnGoPortUnbound(IGoPort goPort)
        {
            SetPortLEDState(goPort, false);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                if (disposing)
                {
                    foreach (var led in _portLEDs)
                    {
                        if (led != null)
                            led.Dispose();
                    }
                }
            }
            _disposed = true;
        }

        ~GoHub()
        {
            Dispose(false);
        }
    }
}
