using System;
using System.Collections;
#if NETMF
using Microsoft.SPOT.Hardware;
#else
using System.Collections.Generic;
#endif

namespace GoBusExperimental
{
    // This class contains all of our board-specific code

    public static class GoBus
    {
        static GoBus()
        {
            // Set up the device-specific root hub
            GoHub hub = new GoHub();

#if NETDUINOGO
            hub.AddGoPort(new GoPort(1, (Cpu.Pin)0x20, (Cpu.Pin)0x1B, SPI.SPI_module.SPI1, (Cpu.Pin)0x00, (Cpu.Pin)0x01), Cpu.Pin.GPIO_NONE, (Cpu.Pin)0x16);
            hub.AddGoPort(new GoPort(2, (Cpu.Pin)0x21, (Cpu.Pin)0x2B, SPI.SPI_module.SPI1, (Cpu.Pin)0x00, (Cpu.Pin)0x01), Cpu.Pin.GPIO_NONE, (Cpu.Pin)0x17);
            hub.AddGoPort(new GoPort(3, (Cpu.Pin)0x22, (Cpu.Pin)0x04, SPI.SPI_module.SPI1, (Cpu.Pin)0x00, (Cpu.Pin)0x01), Cpu.Pin.GPIO_NONE, (Cpu.Pin)0x18);
            hub.AddGoPort(new GoPort(4, (Cpu.Pin)0x23, (Cpu.Pin)0x08, SPI.SPI_module.SPI1, (Cpu.Pin)0x00, (Cpu.Pin)0x01), Cpu.Pin.GPIO_NONE, (Cpu.Pin)0x19);
            hub.AddGoPort(new GoPort(5, (Cpu.Pin)0x24, (Cpu.Pin)0x10, SPI.SPI_module.SPI2, (Cpu.Pin)0x02, (Cpu.Pin)0x03), Cpu.Pin.GPIO_NONE, (Cpu.Pin)0x26);
            hub.AddGoPort(new GoPort(6, (Cpu.Pin)0x25, (Cpu.Pin)0x11, SPI.SPI_module.SPI2, (Cpu.Pin)0x02, (Cpu.Pin)0x03), Cpu.Pin.GPIO_NONE, (Cpu.Pin)0x27);
            hub.AddGoPort(new GoPort(7, (Cpu.Pin)0x2C, (Cpu.Pin)0x32, SPI.SPI_module.SPI2, (Cpu.Pin)0x02, (Cpu.Pin)0x03), Cpu.Pin.GPIO_NONE, (Cpu.Pin)0x28);
            hub.AddGoPort(new GoPort(8, (Cpu.Pin)0x1A, (Cpu.Pin)0x1C, SPI.SPI_module.SPI2, (Cpu.Pin)0x02, (Cpu.Pin)0x03), Cpu.Pin.GPIO_NONE, (Cpu.Pin)0x29);
#endif
#if NETDUINO3WIFI
            hub.AddGoPort(new GoPort(1, (Cpu.Pin)0x3D, (Cpu.Pin)0x30, SPI.SPI_module.SPI4, (Cpu.Pin)0x38, (Cpu.Pin)0x39), (Cpu.Pin)0x37, (Cpu.Pin)0x49);
            hub.AddGoPort(new GoPort(2, (Cpu.Pin)0x3E, (Cpu.Pin)0x31, SPI.SPI_module.SPI4, (Cpu.Pin)0x48, (Cpu.Pin)0x47), (Cpu.Pin)0x3A, (Cpu.Pin)0x4B);
            hub.AddGoPort(new GoPort(3, (Cpu.Pin)0x3F, (Cpu.Pin)0x32, SPI.SPI_module.SPI4, (Cpu.Pin)0x41, (Cpu.Pin)0x40), (Cpu.Pin)0x3C, (Cpu.Pin)0x4E);
#endif

            AddHub(hub);
        }

        private static uint _spiEnumerationClockRate = 500;
        /// <summary>
        /// Represents the default SPI enumeration clock rate (in kHz).
        /// </summary>
        public static uint SPIEnumerationClockRate
        {
            get { return _spiEnumerationClockRate; }
            set { _spiEnumerationClockRate = value; }
        }

#if NETMF
        private static readonly ArrayList _hubs = new ArrayList();
        public static IGoHub[] Hubs { get { return (IGoHub[])_hubs.ToArray(typeof(IGoHub)); } }
#else
        private static readonly List<IGoHub> _hubs = new List<IGoHub>();
        public static IGoHub[] Hubs { get { return _hubs.ToArray(); } }
#endif

        public static IGoHub RootHub
        {
            get
            {
                if (_hubs.Count > 0)
                    return (IGoHub)_hubs[0];
                return null;
            }
        }

        public static void AddHub(IGoHub hub)
        {
            _hubs.Add(hub);
        }

        public static void RemoveHub(IGoHub hub)
        {
            _hubs.Remove(hub);
        }
    }
}
