using System;

namespace GoBusExperimental
{
    public abstract class GoModule : IDisposable
    {
        bool _disposed;

        public abstract Guid ModuleGuid { get; }
        public virtual uint SPIClockRate { get { return GoBus.SPIEnumerationClockRate; } }

        protected readonly IGoPort _goPort;
        public IGoPort GoPort { get { return _goPort; } }

        /// <summary>
        /// Initializes a GoModule connected to an automatically-detected port.
        /// </summary>
        public GoModule()
        {
            foreach (var hub in GoBus.Hubs)
            {
                foreach (IGoPort port in hub.GetGoPortsForModuleGuid(ModuleGuid, false))
                {
                    // Take the first matching IGoPort
                    _goPort = port;
                    Initialize();
                    return;
                }
            }

            throw new Exception();
        }

        /// <summary>
        /// Initializes a GoModule connected to the specified port.
        /// </summary>
        /// <param name="goPort">The port the module is connected to.</param>
        public GoModule(IGoPort goPort)
        {
            _goPort = goPort;
            Initialize();
        }

        protected virtual void Initialize()
        {
            _goPort.Bind(this);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                if (disposing)
                {
                    _goPort.Unbind();
                }
            }
            _disposed = true;
        }

        ~GoModule()
        {
            Dispose(false);
        }
    }
}
