﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Shapes;

namespace SampleApp.Controls
{
    public class CircleGauge : ContentPresenter
    {
        Path _path;
        PathFigure _pathFigure;
        ArcSegment _arcSegment;

        public CircleGauge()
        {
            // Set up content for this control
            _path = new Path();
            _path.StrokeStartLineCap = PenLineCap.Round;
            _path.StrokeEndLineCap = PenLineCap.Round;
            var pathGeometry = new PathGeometry();
            _path.Data = pathGeometry;
            _pathFigure = new PathFigure();
            pathGeometry.Figures.Add(_pathFigure);
            _arcSegment = new ArcSegment();
            _arcSegment.SweepDirection = SweepDirection.Clockwise;
            _pathFigure.Segments.Add(_arcSegment);
            Content = _path;

            SizeChanged += CircleGauge_SizeChanged;
        }

        #region Dependency Properties

        public double StartPositionDegrees
        {
            get { return (double)GetValue(StartPositionDegreesProperty); }
            set { SetValue(StartPositionDegreesProperty, value); }
        }

        public static readonly DependencyProperty StartPositionDegreesProperty =
            DependencyProperty.Register("StartPositionDegrees", typeof(double), typeof(CircleGauge), new PropertyMetadata(0d, UpdateDisplay));


        public double EndPositionDegrees
        {
            get { return (double)GetValue(EndPositionDegreesProperty); }
            set { SetValue(EndPositionDegreesProperty, value); }
        }

        public static readonly DependencyProperty EndPositionDegreesProperty =
            DependencyProperty.Register("EndPositionDegrees", typeof(double), typeof(CircleGauge), new PropertyMetadata(0d, UpdateDisplay));

        public double Value
        {
            get { return (double)GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }

        public static readonly DependencyProperty ValueProperty =
            DependencyProperty.Register("Value", typeof(double), typeof(CircleGauge), new PropertyMetadata(1d, UpdateDisplay));

        public Brush Stroke
        {
            get { return (Brush)GetValue(StrokeProperty); }
            set { SetValue(StrokeProperty, value); }
        }

        public static readonly DependencyProperty StrokeProperty =
            DependencyProperty.Register("Stroke", typeof(Brush), typeof(CircleGauge), new PropertyMetadata(null));

        public double StrokeThickness
        {
            get { return (double)GetValue(StrokeThicknessProperty); }
            set { SetValue(StrokeThicknessProperty, value); }
        }

        public static readonly DependencyProperty StrokeThicknessProperty =
            DependencyProperty.Register("StrokeThickness", typeof(double), typeof(CircleGauge), new PropertyMetadata(15d, UpdateDisplay));

        #endregion

        private void CircleGauge_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            UpdateDisplay();
        }

        private static void UpdateDisplay(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var gauge = d as CircleGauge;
            if (gauge == null)
                return;
            gauge.UpdateDisplay();
        }

        private void UpdateDisplay()
        {
            double start = StartPositionDegrees;
            double end = EndPositionDegrees;

            // Determine adjusted size (minus stroke thickness)
            double strokeThickness = StrokeThickness;
            double adjustedWidth = ActualWidth - strokeThickness;
            double adjustedHeight = ActualHeight - strokeThickness;
            if (adjustedWidth <= 0 || adjustedHeight <= 0)
                return;

            // Get line distance (degrees)
            if (start >= 180)
                start -= 360;
            double distance = (end - start) % 360;
            // Adjust the distance for the value (applied as a percentage)
            distance *= Value;
            // Determine the end angle based on our new distance
            end = (start + distance) % 360;
            // Is this a large arc?
            bool isLargeArc = (distance >= 180);

            // Apply new values
            _pathFigure.StartPoint = DegreesToPoint(start, adjustedWidth, adjustedHeight);
            _arcSegment.Point = DegreesToPoint(end, adjustedWidth, adjustedHeight);
            _arcSegment.IsLargeArc = isLargeArc;
            _arcSegment.Size = new Size(adjustedWidth / 2, adjustedHeight / 2);
            _path.StrokeThickness = StrokeThickness;
            _path.Stroke = Stroke;
        }

        private Point DegreesToPoint(double degrees, double width, double height)
        {
            double w = width / 2;
            double h = height / 2;
            double st = StrokeThickness / 2;
            double rad = (degrees - 90) * Math.PI / 180;
            return new Point(Math.Cos(rad) * w + w + st, Math.Sin(rad) * h + h + st);
        }
    }
}
