﻿using GoBusExperimental;
using Komodex.NETMF;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace SampleApp
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();

            Loaded += MainPage_Loaded;
        }

        private Potentiometer _potentiometer;
        private RgbLed _rgbLed;
        private GoBusExperimental.Button _button;
        private SevenSegmentDisplay _display;

        private async void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            Loaded -= MainPage_Loaded;

            // Initialize root hub
            GoHub hub = (GoHub)GoBus.RootHub;
            hub.AddGoPort(await GoPort.GetGoPortAsync(1, 0, "SPI0", 0), 1);
            hub.AddGoPort(await GoPort.GetGoPortAsync(2, 5, "SPI0", 1, 12), 6);
            hub.AddGoPort(await GoPort.GetGoPortAsync(3, 16, "SPI0", 1, 18), 26);
            hub.AddGoPort(await GoPort.GetGoPortAsync(4, 24, "SPI0", 1, 23), 27);

            // Attempt to connect to the different modules
            var greenBrush = new SolidColorBrush(Colors.Green);
            try
            {
                _potentiometer = new Potentiometer();
                PotentiometerEllipse.Fill = greenBrush;
                await Task.Delay(1);
            }
            catch { }

            try
            {
                _rgbLed = new RgbLed();
                RgbLedEllipse.Fill = greenBrush;
                await Task.Delay(1);
            }
            catch { }

            try
            {
                _button = new GoBusExperimental.Button();
                ButtonEllipse.Fill = greenBrush;
                await Task.Delay(1);
            }
            catch { }

            try
            {
                _display = new SevenSegmentDisplay();
                SevenSegmentDisplayEllipse.Fill = greenBrush;
                await Task.Delay(1);
            }
            catch { }

            RunLoop();
        }

        // Note: the run loop should be implemented as a background thread so SPI communication doesn't interfere with the UI thread.
        // TODO for a future update.
        private async void RunLoop()
        {
            var buttonOffBrush = new SolidColorBrush(Colors.Gray);
            var buttonOnBrush = new SolidColorBrush(Colors.White);

            while (true)
            {
                if (_potentiometer != null)
                {
                    var value = _potentiometer.GetValue();
                    PotentiometerGauge.Value = value;
                    PotentiometerText.Text = (int)(value * 100) + "%";
                    await Task.Delay(1);
                }

                if (_rgbLed != null)
                {
                    byte r = (byte)(RedSlider.Value * 2.55d);
                    byte g = (byte)(GreenSlider.Value * 2.55d);
                    byte b = (byte)(BlueSlider.Value * 2.55d);
                    _rgbLed.SetColor(r, g, b);
                    await Task.Delay(1);
                }

                if (_button != null)
                {
                    if (_button.IsPressed)
                        ButtonPressedEllipse.Fill = buttonOnBrush;
                    else
                        ButtonPressedEllipse.Fill = buttonOffBrush;
                    await Task.Delay(1);
                }

                if (_display != null)
                {
                    _display.SetValue(SevenSegmentDisplayText.Text);
                    await Task.Delay(1);
                }

                await Task.Delay(10);
            }
        }
    }
}
