﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware;
using SecretLabs.NETMF.Hardware.Netduino;
using GoBusExperimental;
using Komodex.NETMF;

namespace SampleApp
{
    public class Program
    {
        public static void Main()
        {
            // Uncomment the demo you want to try

            NP2BreakoutDemo();
            //PWMHubDemo();
            //ExternalHubDemo();
        }

        public static void NP2BreakoutDemo()
        {
            // Create the root hub
            var hub = new GoHub();
            // Set up the port, specifying the pins and SPI bus it uses
            var port = new GoPort(1, Pins.GPIO_PIN_D1, Pins.GPIO_PIN_D0, SPI.SPI_module.SPI1, Cpu.Pin.GPIO_NONE, Cpu.Pin.GPIO_NONE);
            // Add the port to our hub (without specifying a power pin, but using the onboard LED for indication of a bound module)
            hub.AddGoPort(port, Cpu.Pin.GPIO_NONE, Pins.ONBOARD_LED);
            // Add the hub to the static GoBus class (for auto-enumeration)
            GoBus.AddHub(hub);
            // Initialize the display with automatic enumeration
            var display = new SevenSegmentDisplay();
            // Set the value to be displayed
            display.SetValue(1234);
        }

        public static void PWMHubDemo()
        {
            // Get the default root hub object
            var oldHub = (GoHub)GoBus.RootHub;
            // Release control of power and LED output pins
            oldHub.Dispose();
            // Remove the default hub from the list of hubs for automatic enumeration
            GoBus.RemoveHub(oldHub);

            // Create the new hub and add the ports for this board (copied from the Netduino 3 Wi-Fi initialization procedure)
            var hub = new PWMHub();
            hub.AddGoPort(new GoPort(1, (Cpu.Pin)0x3D, (Cpu.Pin)0x30, SPI.SPI_module.SPI4, (Cpu.Pin)0x38, (Cpu.Pin)0x39), (Cpu.Pin)0x37, (Cpu.Pin)0x49);
            hub.AddGoPort(new GoPort(2, (Cpu.Pin)0x3E, (Cpu.Pin)0x31, SPI.SPI_module.SPI4, (Cpu.Pin)0x48, (Cpu.Pin)0x47), (Cpu.Pin)0x3A, (Cpu.Pin)0x4B);
            hub.AddGoPort(new GoPort(3, (Cpu.Pin)0x3F, (Cpu.Pin)0x32, SPI.SPI_module.SPI4, (Cpu.Pin)0x41, (Cpu.Pin)0x40), (Cpu.Pin)0x3C, (Cpu.Pin)0x4E);
            GoBus.AddHub(hub);

            // Try to instantiate a few modules
            try
            {
                var rgbLed = new RgbLed();
            }
            catch { }
            try
            {
                var btn = new GoBusExperimental.Button();
            }
            catch { }
            try
            {
                var pot = new Potentiometer();
            }
            catch { }
            try
            {
                var buzzer = new PiezoBuzzer();
            }
            catch { }
            try
            {
                var disp = new SevenSegmentDisplay();
            }
            catch { }
        }

        public static void ExternalHubDemo()
        {
            // Note: this demo requires custom external hub hardware and custom STM8 firmware.
            var externalHub = new ExternalHub();
            var hubPort = new ExternalHubGoPort(1, Cpu.Pin.GPIO_NONE, Cpu.Pin.GPIO_NONE, SPI.SPI_module.SPI4, Cpu.Pin.GPIO_NONE, Cpu.Pin.GPIO_NONE);
            externalHub.AddGoPort(hubPort);
            hubPort = new ExternalHubGoPort(2, Cpu.Pin.GPIO_NONE, Cpu.Pin.GPIO_NONE, SPI.SPI_module.SPI4, Cpu.Pin.GPIO_NONE, Cpu.Pin.GPIO_NONE);
            externalHub.AddGoPort(hubPort);
            hubPort = new ExternalHubGoPort(3, Cpu.Pin.GPIO_NONE, Cpu.Pin.GPIO_NONE, SPI.SPI_module.SPI4, Cpu.Pin.GPIO_NONE, Cpu.Pin.GPIO_NONE);
            externalHub.AddGoPort(hubPort);
            GoBus.AddHub(externalHub);

            var disp = new SevenSegmentDisplay();
            disp.SetValue(1234);
        }
    }
}
